//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#ifndef __BSP_H
#define __BSP_H

/**
 * PICtail to PIC16LF1938 connections, with SX1239 PICtail.
 * <table>
 * <tr><th>PICtail Pin  </th><th>PIC16F1938 pin </td><th>SX1243 PICtail signal</th></tr>
 * <tr><td>2            </td><td>NC             </td><td>RESET                  </td></tr>
 * <tr><td>17           </td><td>RB5            </td><td>RESET                  </td></tr>
 * <tr><td>8            </td><td>RC5            </td><td>MOSI                   </td></tr>
 * <tr><td>10           </td><td>RC4            </td><td>MISO                   </td></tr>
 * <tr><td>12           </td><td>RC3            </td><td>SCL                    </td></tr>
 * <tr><td>19           </td><td>RB4            </td><td>DIO0                   </td></tr>
 * <tr><td>3            </td><td>RA1            </td><td>DIO1                   </td></tr>
 * <tr><td>21           </td><td>RB3            </td><td>DIO1                   </td></tr>
 * <tr><td>23           </td><td>RB2            </td><td>DIO2                   </td></tr>
 * <tr><td>24           </td><td>RC2            </td><td>nSS                    </td></tr>
 * <tr><td>25           </td><td>RB1            </td><td>NC                     </td></tr>
 * <tr><td>1            </td><td>RA2            </td><td>NC                     </td></tr>
 * <tr><td>4            </td><td>RA3            </td><td>NC                     </td></tr>
 * </table>
 *
 * SAP to PIC16LF1938 connections
 * <table>
 * <tr><th>SAP Pin  </th><th>SAP Signal </th><th>Micro signal   </th></tr>
 * <tr><td>6*       </td><td>RX         </td><td>RC7            </td></tr>
 * <tr><td>6*       </td><td>SDO        </td><td>RC5            </td></tr>
 * <tr><td>5        </td><td>SCL/SCK    </td><td>RC3            </td></tr>
 * <tr><td>4        </td><td>SDA/SDI    </td><td>RC4            </td></tr>
 * <tr><td>1        </td><td>TX/SS      </td><td>RC6            </td></tr>
 * </table>
 *
 * PIC16LF1938 connections by pin. Peripherals of interest noted where appropriate.
 * <table>
 * <tr><th>Pin          </th><th>Signal                                         </th></tr>
 * <tr><td>RA0          </td><td>Test point                                     </td></tr>
 * <tr><td>RA1          </td><td>PICtail connector pin 3                        </td></tr>
 * <tr><td>RA2          </td><td>PICtail connector pin 1                        </td></tr>
 * <tr><td>RA3          </td><td>PICtail connector pin 4                        </td></tr>
 * <tr><td>RA4          </td><td>LED D4                                         </td></tr>
 * <tr><td>RA5          </td><td>LED D5                                         </td></tr>
 * <tr><td>RA6          </td><td>LED D7                                         </td></tr>
 * <tr><td>RA7          </td><td>LED D6                                         </td></tr>
 * <tr><td>RB0/AN12     </td><td>Pushbuttons                                    </td></tr>
 * <tr><td>RB1          </td><td>PICtail connector pin 25                       </td></tr>
 * <tr><td>RB2          </td><td>PICtail connector pin 23                       </td></tr>
 * <tr><td>RB3          </td><td>PICtail connector pin 21                       </td></tr>
 * <tr><td>RB4          </td><td>PICtail connector pin 19                       </td></tr>
 * <tr><td>RB5          </td><td>PICtail connector pin 17                       </td></tr>
 * <tr><td>RB6/ICSPCLK  </td><td>PIC-to-PIC comm, In-circuit debugging          </td></tr>
 * <tr><td>RB7/ICSPDAT  </td><td>PIC-to-PIC comm, In-circuit debugging          </td></tr>
 * <tr><td>RC0/T1OSO    </td><td>Timer 1 oscillator                             </td></tr>
 * <tr><td>RC1/T1OSI    </td><td>Timer 1 oscillator                             </td></tr>
 * <tr><td>RC2          </td><td>PICtail connector pin 24                       </td></tr>
 * <tr><td>RC3/SCL/SCK  </td><td>SAP connector pin 5, PICtail connector pin 12  </td></tr>
 * <tr><td>RC4/SDA/SDI  </td><td>SAP connector pin 4, PICtail connector pin 10  </td></tr>
 * <tr><td>RC5/SDO      </td><td>SAP connector pin 6, PICtail connector pin 8   </td></tr>
 * <tr><td>RC6/TX       </td><td>SAP connector pin 1 (note)                     </td></tr>
 * <tr><td>RC7/RX       </td><td>SAP connector pin 6 (note)                     </td></tr>
 * <tr><td>RE3/MCLR     </td><td>PIC-to-PIC comm, In-circuit debugging          </td></tr>
 * </table>
 */

#include <xc.h>

#define _XTAL_FREQ 16000000


//PICtail SPI lines
#define	PICtail_SS              LATCbits.LATC2
#define	PICtail_SCK             LATCbits.LATC3
#define	PICtail_MISO            LATCbits.LATC4
#define	PICtail_MOSI            LATCbits.LATC5

#define TRIS_PICtail_SS         TRISCbits.TRISC2
#define TRIS_PICtail_SCK        TRISCbits.TRISC3
#define TRIS_PICtail_MISO       TRISCbits.TRISC4
#define TRIS_PICtail_MOSI       TRISCbits.TRISC5

#define SX1239_RESET            LATBbits.LATB5
#define SX1239_RESET_DIR        TRISBbits.TRISB5

//SAP
#define TRIS_SAP_RX             TRISCbits.TRISC7
#define TRIS_SAP_TX             TRISCbits.TRISC6

#define SAP_RX                  LATCbits.LATC7
#define SAP_TX                  LATCbits.LATC6

//SPI To SLAVE device (PIC-TO-PIC)
#define MISO                    PORTEbits.RE3       // Also nMCLR
#define MOSI                    LATBbits.LATB7      // Also ICSPDAT
#define SCK                     LATBbits.LATB6      // Also ICSPCLK

#define TRIS_MISO               TRISEbits.TRISE3
#define TRIS_MOSI               TRISBbits.TRISB7
#define TRIS_SCK                TRISBbits.TRISB6

//GPIO from PICtail
#define PICtail_RAW_DATA                PORTBbits.RB2

#define TRIS_PICtail_RAW_DATA           TRISBbits.TRISB2

//LEDS 
#define LED1            LATAbits.LATA4
#define LED2            LATAbits.LATA5
#define LED3            LATAbits.LATA6
#define LED4            LATAbits.LATA7
#define BUTTONS         PORTAbits.RA0 /**@brief SW1->4 multiplixed on single ADC pin*/
#define LED_ON          0
#define LED_OFF         1
// hardware related definitions
#define LEARN_LED       LED1


#define TRIS_LED1       TRISAbits.TRISA4
#define TRIS_LED2       TRISAbits.TRISA5
#define TRIS_LED3       TRISAbits.TRISA6
#define TRIS_LED4       TRISAbits.TRISA7
#define TRIS_BUTTONS    TRISBbits.TRISB0 /**@brief SW1->4 multiplixed on single ADC pin*/

#define ANSEL_BUTTONS   ANSELBbits.ANSB0

//#define TRIS_RXOUT      TRISAbits.TRISA3
//#define RXOUT           LATAbits.LATA3 // jason

#endif