//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/**
 * @file
 * @brief This file contains functions related to the push buttons. 
 *
 * @defgroup buttons Buttons
 * @ingroup master
 * @{
 */

#include <xc.h>
#include "bsp.h"
#include "buttons.h"

//button_state buttons_state;
/**
 * Take a reading
 * Resistor Voltages @3V:
 * R7->R10
 * 2.97 - 2.67  - 2.5 - 2.34
 *
 * Set Thresholds at:
 * >2.8 : 2.8-2.6 : 2.59-2.40  : >2.15
 */
button_state buttons_update(void)
{
    uint16_t reading;
    //start the ADC
    reading = adc_read();

  //  if(reading >956)
  //      return SW1;
  //  if(reading >887)
    //    return SW2;
  //  if(reading >819)
    //    return SW3; // jason. no need for these
    if(reading >734)
        return SW4;
    else
        return NONE;
}

/**
 * @brief Configures buttons
 */
void buttons_init(void)
{
    WPUBbits.WPUB0 = 0;
    TRIS_BUTTONS = 1; //input
    ANSEL_BUTTONS = 1; //analog
    ADCON0bits.CHS = 0b01100;
    ADCON1 = 0b11010000; //Right justified using Vdd/Vss as reference voltages :: FOSC/16 speed
    ADCON0bits.ADON = 1;
    __delay_ms(2); //delay for charging cap
}

uint16_t adc_read(void){
    uint16_t voltage;

    di(); //Turn off interrupts
    GO = 1; //start the ADC
    while (GO) continue; //Wait for conversion to complete
    voltage = ADRESH; //grab the top 2 MSBs
    voltage = voltage << 8; //make room for lower 8 bits
    voltage |= ADRESL;
    ei(); //enable interrupts

    return voltage;
}

/** @}*/
