//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include <stdbool.h>
#include "includes.h"

/** Buffer for sprintf calls. */
uint8_t buffer7[22];

/**
 * Decrypts and interprets received codes
 */
void Remote(void) {
    // a frame was received and is stored in the receive buffer
    // move it to decryption Buffer, and restart receiving again

    // Copy the received data to the work buffer
    memcpy(Buffer, RX_Buffer, sizeof(RX_Buffer));

    switch (RFtype)
    {
        case KEELOQ_PACKET: // Standard Keeloq frame detected
            NormalKeyGen(); // Compute the decryption key
            Decrypt(); // Decrypt the hopping code portion

            // Validate received packet (aka, is it a paired device?)
            if (Find(RFtype) == FALSE)
            {   // Transmitter is not paired. Are we in learn mode?
                if (FLearn)
                {   // look for new space to store record
                    if (!Insert())
                    {   // fail if no memory available
                        break;
                    }
                    else
                    {
                        // ASSERT Ind is pointing to a valid memory location
                        /// JASON, PUT IND ADDRESS HERE WHEN SEED = 0; 
                        
                        IDWrite(); // write Serial Number in memory
                        FHopOK = TRUE; // enable updating Hopping Code
                        HopUpdate(); // Write Hoping code in memory

                        CFlash = 10; // request Led flashing to indicate successful learning
                        CTFlash = TFLASH; // load period timer
                       // LED1 = TRUE; // start with Led on
                        FLearn = FALSE; // terminate successfully Learn
                        CTLearn = 0;
                    }
                }
                else
                {
                    p2p_lcd_clear();
                    p2p_lcd_goto(0,0);
                    p2p_lcd_puts("KLQ Transmitter");
                    p2p_lcd_goto(0,1);
                    p2p_lcd_puts("Not Learned! ");
                }
            }
            else if (DecCHK() == FALSE)
            {   //Transmitter is paired, but message is not valid
                p2p_lcd_clear();
                p2p_lcd_goto(0,0);
                p2p_lcd_puts("Not a Valid KLQ");
                p2p_lcd_goto(0,1);
                p2p_lcd_puts("Transmitter");
            }
            else
            {   // Transmitter is already paired
                // Check Hopping Code integrity; are we within the acceptable window?
                if (!HopCHK())
                {
                    p2p_lcd_clear();
                    p2p_lcd_goto(0,0);
                    p2p_lcd_puts("Hop Check");
                    p2p_lcd_goto(0,1);
                    p2p_lcd_puts("     Fail");
                    break;
                }

                if (FSame) // identified same hopping code as last memorized
                {
                    if (COut > 0) // if output is still active
                        COut = TOUT; // reload timer to keep active
                    else
                        return; // else discard
                }
                else
                {   // hopping code incrementing properly
                    HopUpdate(); // update memory
                   // LED1 = ON; //Set an Output. Note: thsi can be set according to function code
                    if (((Buffer[7] ^ FCode) & 0xf0) == 0) // check against learned function code
                        LED2 = OFF;
                    
                    if(SEED[0] == 0x1B) /// jason jason jason; signal to ose
                    {  /// 0X1B MEANS IT IS A BOTTOM SLAT TRANSMITTER, NOT A HANDSET
                        LED1 = ON;
                    }
                    if(SEED[0]!= 0x1B)
                    {
                    LED3 = ON; // JASON JASON JASON signal to main chip; changed from OFF; IT GOES HIGH ON CORRECT SIGNAL NOW
                    }
                    
                    COut = TOUT; // Init LED output timer
                } // recognized

                p2p_lcd_clear();
                p2p_lcd_goto(0, 0);
                sprintf(&buffer7, "KLQ: %01X%02X%02X%02X",
                    SEED[3], SEED[2], SEED[1], SEED[0]);
                p2p_lcd_puts(&buffer7);

                p2p_lcd_goto(0, 1);
                sprintf(&buffer7, "C:%01X%02X%02X%02X F:%1X",
                    Buffer[3] & 0xF, Buffer[2], Buffer[1], Buffer[0],
                    Buffer[3] >> 4);
                p2p_lcd_puts(&buffer7);
            }

            break;

        case AES_PACKET: // AES frame detected
            AESKeyGen();

            if (Find(RFtype) == FALSE)
            {
                p2p_lcd_clear();
                p2p_lcd_goto(0,0);
                p2p_lcd_puts("AES Transmitter");
                p2p_lcd_goto(0,1);
                p2p_lcd_puts("Not Learned! ");
            }
            else if (AES_DecCHK() == FALSE)
            {
                p2p_lcd_clear();
                p2p_lcd_goto(0,0);
                p2p_lcd_puts("Not a valid AES");
                p2p_lcd_goto(0,1);
                p2p_lcd_puts("Transmitter");
            }
            else
            {
                p2p_lcd_clear();
                p2p_lcd_goto(0,0);

                sprintf(&buffer7, "AES: %02X%02X%02X%02X",
                    Buffer[4], Buffer[5], Buffer[6], Buffer[7]);
                p2p_lcd_puts(&buffer7);

                p2p_lcd_goto(0,1);
                sprintf(&buffer7, "C:%02X%02X%02X%02X F:%02X%02X",
                    Buffer[12], Buffer[13], Buffer[14], Buffer[15],
                    Buffer[2], Buffer[3]);
                p2p_lcd_puts(&buffer7);
            }

            break;

        case XTEA_PACKET: // XTEA frame detected
            p2p_lcd_clear();
            p2p_lcd_goto(0,0);
            p2p_lcd_puts("XTEA not");
            p2p_lcd_goto(0,1);
            p2p_lcd_puts("supported");

            break;
    }

    // Ready to receive a new frame
    RFFull = FALSE;

} // remote


