//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include "includes.h"

//--------------------------------------------------------------------
 //EERead ()//
//--------------------------------------------------------------------
uint8_t EERead(uint8_t EE_Adress)
{
#if defined(HI_TECH_C) || defined(__XC)
    return eeprom_read(EE_Adress);
#else
	uint8_t EE_Data;
	EEPGD = 0;
	EEADR = EE_Adress;
	RD	= 1;
	EE_Data = EEDAT;	
	return EE_Data;
#endif
}

//--------------------------------------------------------------------
// EEWrite ()
//--------------------------------------------------------------------
uint8_t EEWrite(uint8_t EE_Adress, uint8_t EE_Data)
{
#if defined(HI_TECH_C) || defined(__XC)
    eeprom_write(EE_Adress, EE_Data);
#else
	EEPGD = 0;			// Set to write EEPROM area
	EEDAT = EE_Data;		// Load Data
	EEADR = EE_Adress;	// Load Address
	
	GIE	= 0;				// Turn-off Global Interrupts
	GIE = 0;
	WREN = 1;			// Enable writes
	EECON2 = 0x55;		// Sequence necessary to unlock EEPROM
	EECON2 = 0xAA;
	WR = 1;				// Start to write

	while(WR);			// Wait for EEPROM write to complete
#endif
	return TRUE;
}

//--------------------------------------------------------------------
// RDWord ()
//--------------------------------------------------------------------
void RDword(uint16_t Ind)
{
    Dato = EERead(Ind);
    Dato += (uint16_t) EERead( Ind+1) <<8;
}

//--------------------------------------------------------------------
// RDnext ()
//--------------------------------------------------------------------
void RDnext(void)
{
    // Continue reading
    EEADR++;        // NOTE generate no carry
    Dato = ((RD=1), EEDATA);
    EEADR++;
    Dato += ((RD=1), EEDATA)<<8;
}

//--------------------------------------------------------------------
// WRword ()
//--------------------------------------------------------------------
void WRword(uint16_t Ind)
{

    
    EEWrite( Ind, Dato); 
	GIE = 1; // write and re-enable interrupt
    EEWrite( Ind+1, Dato>>8); // shift 8 because in hex
	GIE = 1; 
}

/**
 * Search through the whole table the given a record whose ID match
 *
 * Searches nonvolatile memory for a serial number matching IDHi, IDMi, IDLo.
 * If found, updates Ind with the address of the record, EHop with the sync
 * counter value, and ETemp with a second copy of the sync counter.
 *
 * \pre IDHi, IDMi, IDLo loaded with serial number to search
 *
 * \return TRUE if matching record found
 */
uint8_t Find(uint8_t packetType)
{
    uint8_t Found = FALSE;      // init to not found

    if (packetType == AES_PACKET)
    {
        Found = TRUE;
    }
    else if (packetType == KEELOQ_PACKET)
    {
        for (Ind=0; Ind < (EL_SIZE * MAX_USER); Ind+=EL_SIZE)
        {
            RDword( Ind);       // read first Word // a dato is an 8 bit, and ind - 
            FCode = (Dato>>8);

            if ( (FCode & 0xf0) == 0xf0)		// check if there is a function code // jason, if the contents is 00, then continue
                continue;   // empty

            if (IDLo != (Dato & 0xff))		// ID match? // jason jason jason, not to match? is this the hop part? ? 
                continue;   // fails match

            RDnext();       				// Read Next Word
            if ( ( (Dato & 0xff) == IDMi) && ( (Dato>>8) == IDHi))
            {
                Found = TRUE;     // match
                break;
            }
        }

        if (Found == TRUE)
        {
            RDnext();               // read HopHi/Lo
            EHop = Dato;
            RDnext();               // read HopHi2/Lo2
            ETemp= Dato;
        }
    }
    else
        Found = FALSE;

    return Found;
}

/**
 * Search through the whole table for an empty space. Uses IDHi, IDMi, and IDLo
 * as the serial number to insert and writes to Ind the address of the empty
 * record.
 * @return FALSE if no empty space found
 */
uint8_t Insert(void)
{
    for (Ind=8; Ind < (EL_SIZE * MAX_USER); Ind+=EL_SIZE) // jason jason jason, originally ind=8; ind < (EL etcetc)
    { // so that the first address can be reserved for the safety edge
        RDword(Ind);        // read first Word
        FCode = (Dato>>8);
        // check if 1111xxxx
        if ( (FCode & 0xf0) == 0xf0) 
         
            return TRUE;    // insert point found
    } // for        

    return  FALSE;          // could not find any empty slot
} // Insert

/**
 * Writes IDHi, IDMi, IDLo, and function code to the address indicated by Ind.
 * @return Returns FALSE if FLearn indicates the application is not in learning
 * mode.
 */
uint8_t IDWrite(void)
{
    if (!FLearn) 
        return FALSE;           // Guard statement: check if Learn ON

    Dato = Buffer[7];
    Dato = (Dato<<8) + IDLo;
    
    if(SEED[0] == 0x1B) // jason jason jason --- for the ose command. only for bottom transmitters with this end of serial
    {
        Ind = 0x0;
    }    
    
    WRword(Ind);                // write first word

    Dato = IDHi;
    Dato = (Dato<<8) + IDMi;
    
    if(SEED[0] == 0x1B) // jason jason jason --- for the ose command. only for bottom transmitters with this end of serial
    {
        Ind = 0x0;
    }
    
    WRword(Ind+2);              // write second word
    
    return TRUE;
} // IDWrite
          
/**
 * Update the sync counter pointed to by Ind with the value found in HopHi and
 * HopLo.
 *
 * @return Returns FALSE if FLearn indicates the application is not in learning
 * mode.
 */
uint8_t HopUpdate(void)
{
    if (!FHopOK) 
        return FALSE;           // Guard statement: check if Hop update 

    Hop = ((uint16_t)HopHi<<8) + HopLo;
    Dato = Hop;
    WRword(Ind+4);              // write at offset +4
    Dato = Hop;
    WRword(Ind+6);              // back up copy at offset +6
    FHopOK = FALSE;             // for safety disable updating hopping code
    
    return TRUE;
} // HopUpdate

/**
 * Mark all records free
 * @return Always returns true.
 */
uint8_t ClearMem(void)
{
//    char i;
    for (Ind=8; Ind < (EL_SIZE * MAX_USER); Ind+=EL_SIZE) // jason jason jason, originally Ind=0; ind < etctc
    { // so that the first address can be reserved for the safety edge
        Dato = 0xffff;
        WRword( Ind);
    }
  return TRUE;
} // ClearMem


//--------------------------------------------------------------------
// End of File : Table.C
//--------------------------------------------------------------------
        
