//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/** \file
 *
 * \note This firmware works for all frequency and modulation combinations
 * supported by the SX1239. Ensure that the frequency and modulation settings
 * in sx1239_config.h match your hardware configuration.
 */

//Device Definitions
#include <xc.h>
//memcpy
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>

#include "bsp.h"

//XLP Routines
#include "main.h"
#include "sx1239.h"
#include "spi.h"
#include "buttons.h"
#include <time.h>

//KeeLoq
#include "Global.h"
#include "KeeLoq_RX.h"
#include "KeeLoqAlg.h"
#include "Keygen.h"
#include "Table.h"
#include "keeloq.h"

//Common between PICs
#include "commands.h"
#include "p2p.h"

#include "Config.h"

#if defined(__DEBUG)
// Disable Code Protect if running with the debugger.
#pragma config CPD=OFF, CP=OFF
#else
#pragma config CPD=ON, CP=ON
#endif

#pragma config BOREN=OFF, IESO=OFF, FCMEN=OFF, MCLRE=OFF, FOSC=INTOSC
#pragma config WDTE=OFF, PWRTE=OFF, CLKOUTEN=ON, LVP=OFF, STVREN=ON
#pragma config WRT=OFF, BORV=LO, PLLEN=OFF

//This is where the received bytes will be saved into from the ISR
//----------------- flags defines ------------------------------------
/** Receiver state */
uint8_t RFstate;
/** Timer counter */
int8_t RFcount;
/** Receive buffer pointer */
uint8_t Bptr;
/** Received bits counter */
uint8_t BitCount;
/** Determines what type of encrypted signal was received */
uint8_t RFtype;


//----------------- flags defines ------------------------------------

/** Hopping code verified OK */
bit FHopOK;
/** Same code as previous */
bit FSame;
/** State bit that indicates if learn mode is active */
bit FLearn;
/** Resync required */
bit F2Chance;
/** Flag indicating that the receive buffer is full */
bit RFFull;
/** Sampled RF signal */
bit RFBit;
/** Incoming trasnmission Synch level detect flag */
bit RFsynch;

//----------------- Encrypted Reception buffer -----------------------
/** Decryption Work Buffer */
uint8_t Buffer[22];
/** RF Receive buffer */
uint8_t RX_Buffer[22];

/** seed value = serial number */
uint8_t SEED[4];
/** resync value for 2 Chance */
uint16_t NextHop;
/** output timer */
uint8_t COut;
/** Function codes and upper nibble of serial number */
uint8_t FCode;
/** 16 bit extended timer */
uint16_t XTMR;
/** Learn timers and counter */
uint8_t CLearn, CTLearn;
/** Led flashing timer and counter */
uint8_t CFlash, CTFlash;
/** Temp storage for read and write to mem. */
uint16_t Dato;
/** Address pointer to record in mem. */
uint16_t Ind;
/** Hopping code sync counter */
uint16_t Hop;
uint16_t Hop1;
/** Last value of sync counter (from EEPROM) */
uint16_t EHop;
uint16_t EHop1;
/** Second copy of sync counter */
uint16_t ETemp;
uint16_t ETemp1;
/** Flag set in the ISR to indicate to mainline code that the heartbeat time has elapsed */
volatile bit Continue_Count;

/** Temporary buffer for sprintf/LCD operations */
uint8_t sbuf[22];

//--------------------------------------------------------------------
// Internal Function Prototypes
//--------------------------------------------------------------------
void keeloq(void);
void flash_leds(uint8_t t);
void Delay(uint16_t Count); // Delay function
void Remote(void); // Decodes and determines validity of incoming transmission
void Init_886(void); // General initialization
void tmr1_isr(void); // Timer 1 for general timing

void HDWSelfTests(void);
extern bool flashLEDs;

//--------------------------------------------------------------------

void main(void)
{
    
  
    // init warmup
    main_init();
 //   p2p_init();
    __delay_ms(200);  

    // Configure for KeeLoq data polling
    InitReceiver();

    // Init MSSP1 for SX1239 communication
    pictail_init(); 

    // Init the 4 buttons (SW1->4)
    buttons_init(); 

    // Init comms between PICs
//    p2p_recover();

    // Prepare the LCD
    
//    p2p_lcd_clear();
//    p2p_lcd_goto(0,0);
    

    // Check for hardware test mode. If SW1 is held at start-up, begin test.
 //   if (SW1_HELD)
 //   {
 //       LED1 = LED2 = LED3 = LED_ON;
//        HDWSelfTests();
//        RESET();
//    }

   // p2p_lcd_puts("KeeLoq 3 Demo");

    // Initialize the SX1239
    sx1239_init();
    sx1239_init();
     sx1239_init();   
//    __delay_ms(1000); // JASONJASONJASON I ADDED THIS DELAY, IN ORDER TO GIVE SX TIME TO COMMS

    // Flash LED to indicate that master is ready and entering the main loop
    flash_leds(2);    

    while (1)
    {
        if (RFFull) // RF receive buffer contains a message
        {
            LED2 = LED_ON;
            Remote(); // Get Message
            LED2 = LED_OFF;
            RFFull = 0;
        }

        // This portion of the KeeLoq code expects to be run only every 64 ms.
        // The timer ISR will set the Continue_Count to non-zero at that time.
        if (Continue_Count)
        {
            // We perform the Learn button check and the LED check every 64ms.
            Continue_Count = 0; 

            // Check the Learn button.
            if (LEARN)
            {   // It's pressed. CLearn tracks how long it has been pressed.
                CLearn++;

                // pressing Learn button for more than 10s -> ERASE ALL
                if (CLearn == 156) // 156 * 64 ms = ~10s
                {
                    p2p_lcd_clear();
                    p2p_lcd_goto(0,0);
                    p2p_lcd_puts("Memory");
                    p2p_lcd_goto(0,1);
                    p2p_lcd_puts("Erased");

                    LED2 = LED_OFF;     // switch off Learn Led
                    LED3 = OFF; // jason jason
                    while (LEARN);      // wait for button release
                    LED2 = LED_ON;      // signal Led on
                    LED3 = ON; // jason jason
                    ClearMem();         // erase all command!

                    COut = TOUT;        // single long flash pulse time
                    // timer will switch off Led

                    // Reset variables related to learn mode
                    CLearn = 0;
                    CTLearn = 0;
                    FLearn = FALSE;
                }

                // Require that the learn button be pressed for 1 second before
                // entering learn mode
                if (CLearn == 15) // 15 * 64 ms = ~ 1 s
                {   // Learn button has been held for the required minimum time
                    FLearn = TRUE;      // enter learn mode command
                    CTLearn = TLEARN;   // load timout value
                    LED2 = LED_ON;      // Indicate learn mode active
                    
                    // addition for alerting Main board to learn mode active
                    // Jason 
                    LED3 = ON;

                    p2p_lcd_clear();
                    p2p_lcd_goto(0,0);
                    p2p_lcd_puts("Learn mode");
                    p2p_lcd_goto(0,1);
                    p2p_lcd_puts("active");
                }
            }
            else
            {   // The learn button is not pressed, so reset the counter.
                CLearn = 0;
            }

            // Toggle the --- LED
            if (COut > 0) // if timer running
            {
                COut--;
                if (COut == 0) // when it reach 0
                {
                    LED1 = OFF; // jason added, for ose signal out line
                    LED2 = LED_OFF;
                    LED3 = OFF; // JASON JASON JASON ORIGINALLY OFF
                    // STANDARD SETTING IS 'LED_OFF', WHICH MEANS '1'
                }
            }

            // Check for Learn Mode Timeout.
            if (CTLearn > 0)
            {
                CTLearn--; // count down
                if (CTLearn == 0)
                {   // Time-out period has elapsed
                    LED2 = LED_OFF; // exit Learn mode
                    LED3 = OFF;
                    FLearn = FALSE;

                    p2p_lcd_clear();
                    p2p_lcd_goto(0,0);
                    p2p_lcd_puts("Learn mode");
                    p2p_lcd_goto(0,1);
                    p2p_lcd_puts("timeout");
                }
            }

            // Led Flashing
            if (CFlash > 0)
            {
                LED3 = OFF;
                CTFlash--; // count down
                if (CTFlash == 0) // if timed out
                {
                    CTFlash = TFLASH; // reload timer
                    CFlash--; // count one flash
                    LED2 = LED_OFF; // toggle Led
                    LED3 = ON; // Jason
                    if (CFlash & 1)
                        LED2 = LED_ON;
                    LED3 = ON; // jason jason
            if(CFlash == 0)
            {
                LED3 = OFF;
            }                    
                }
            }

        } // Continue_Count
    }
}

/**
 * General intialization not tied to any specific peripheral or feature.
 *
 * \warning This function sets the system clock source and divider value. The
 * system clock is used as the timebase for all sources in the micro (excepting
 * the RTCC, if used). If you change the system clock speed, you must also
 * verify operation of the timers.
 */
void main_init(void)
{
#if _XTAL_FREQ == 16000000
    OSCCONbits.IRCF = 0b1111;
#else
    #error unknown system clock speed
#endif

    LATC = 0;
    LATB = 0;
    LATA = 0;

LED2 = LED4 = LED_OFF; //NEW LINE - ALL THESE LEDS ARE SET TO '1' (WHICH EQUATES TO OFF)
  // OFF DUE TO HARDWARE WIRING OF LEDS, EVEN THOUGH IT'S '1' HIGH
  //  LED1 = LED2 = LED3 = LED4 = LED_OFF; //ORIGINAL LINE
 //LED1 = LED2 = LED4 = LED_OFF;// this alone is original

LED1 = LED_ON; // jason jason, optical safety edge signal
 LED3 = LED_ON; // JASON added this line, to make LED3 low at all times except in transmission
 // THIS LINE SETS LED3 / PIN 10 /RA6 TO '0', LOW AS STANDARD

    TRIS_LED1 = 0; // status LEDs
    TRIS_LED2 = 1;
    TRIS_LED3 = 0;
    TRIS_LED4 = 1;
//    TRIS_RXOUT = 0; // JASON

   // TRISA3 = 0;

    TRISA = 0x00;
    ANSELA = 0x00;
    ANSELB = 0x00;

    // SX1239 demodulated output as input to PIC
    TRIS_PICtail_RAW_DATA = 1; 

    //TMR2 is used for a 64 ms heartbeat interrupt, used mostly by the KeeLoq
    // receive routines.
#if _XTAL_FREQ == 8000000
    PR2 = 125;
#elif _XTAL_FREQ == 16000000
    PR2 = 250;
#else
    #error Unknown system clock speed
#endif
    T2CONbits.T2CKPS = 0b11; //prescaler of 1:64
    T2CONbits.T2OUTPS = 0b1111; // postscaler of 1:16

    TMR2 = 0x00;
    PIE1bits.TMR2IE = 1;
    PIR1bits.TMR2IF = 0;
    T2CONbits.TMR2ON = 1;
    

    // TMR1 uses 32 kHz external crystal
    T1GCON = 0x00;              // Ensure that T1 Gate is disabled
    T1CONbits.nT1SYNC = 1;      // Do not sync with system clock
    T1CONbits.TMR1CS = 0b10;    // external clock as source
    T1CONbits.T1OSCEN = 1;      // enable oscillator driver
    T1CONbits.T1CKPS = 0;       // 1:1 prescaler
    
    TMR1 = 0;
    PIE1bits.TMR1IE = 1;
    PIR1bits.TMR1IF = 0;
    T1CONbits.TMR1ON = 1;


    
    INTCONbits.PEIE = 1;
    ei();

    TRISBbits.TRISB1 = 0;
}

/**
 * Flashes LEDs
 * \param t Number of times to flash
 */
void flash_leds(uint8_t t)
{
    uint8_t i;
    for (i = 0; i < t; i++) 
    {
LED2 = LED_ON;
        __delay_ms(125);
LED2 = LED_OFF;
        __delay_ms(125);
    }
}

/** Interrupt vector entry point */
__interrupt void ISR(void)
{
    //  TMR0 interrupt for data packet aquisition
    if (T0IF && T0IE)
    {   
        rxi();
    }

    // TMR1 interrupt is derived from the 32 kHz oscillator
    if (TMR1IF && TMR1IE)
    {
        TMR1IF = 0;
      //  LED1 ^= 1;

        if (flashLEDs) 
        {   // Used in hardware test routines.
            LED4 = LED2;
        }
    }

    // TMR2 provides a system 64 ms heartbeat
    if (TMR2IF && TMR2IE)
    {
        TMR2IF = 0;
        Continue_Count = 1;
    }

    if (INTCONbits.IOCIF && IOCIE)
    {
        IOCIF = 0;
    }

}
