//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/**
 * \file
 * Routines for PIC-to-PIC communication
 */

#include <stdint.h>
#include "p2p.h"
#include "bsp.h"

void p2p_init(void)
{
    TRIS_MOSI = 0;
    TRIS_MISO = 1; //input is on RE3....read-only anyways
    TRIS_SCK = 0;

    MOSI = 0;
    SCK = 0;
}

void p2p_lcd_puts(const uint8_t * buf)
{
    p2p_spi(P2P_CMD_LCD_PUTS);

    // Use do-while so that the terminating NULL is sent.
    do
    {
        p2p_spi(*buf);
    } while (*buf++ != 0);
}

void p2p_lcd_clear(void)
{
    p2p_spi(P2P_CMD_LCD_CLEAR);
}

void p2p_lcd_goto(uint8_t x, uint8_t y)
{
    p2p_spi(P2P_CMD_LCD_GOTO);
    p2p_spi(x);
    p2p_spi(y);
}

void p2p_lcd_putch(uint8_t c)
{
    p2p_spi(P2P_CMD_LCD_PUTCH);
    p2p_spi(c);
}

void p2p_spi(uint8_t data)
{
    unsigned char i;
    //shift data with MSB first

    __delay_ms(1);
    while (MISO == 1);

    for (i = 8; i != 0; i--)
    {
        if (data & 0x80)
            MOSI = 1;
        else
            MOSI = 0;

        //__delay_us(PIC_PIC_DELAY);
        SCK = 1;
        __delay_us(PIC_PIC_DELAY);
        SCK = 0;

        data <<= 1;
    }
}

void p2p_recover(void)
{
    unsigned char i;
    for (i = 0; i < 10; i++)
    {
        p2p_spi(0xFF);
    }
}
