//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/**
 * @file pictail.c
 * @brief
 */
#include <xc.h>
#include "bsp.h"
#include "sx1239_config.h"

/**
 * @brief Initilizes the SPI BUS for communication to the SX1239
 */
void pictail_init(void) {

    // SS =  RC1 ||  SCK = RC3 ||  MISO = RC4 || MOSI = RC5

    TRIS_PICtail_SS = 0;
    TRIS_PICtail_SCK = 0;
    TRIS_PICtail_MOSI = 0;
    TRIS_PICtail_MISO = 1;

    SSPCON1 = 0;
    SSPCON2 = 0;
    SSPSTAT = 0;

    SSPSTAT = 0x00;
    SSPCON1bits.CKP = 0; //idle state for clock is a low level
    SSPSTATbits.CKE = 1; //transmit occurs from idle to active state
    SSPSTATbits.SMP = 0; //input data sampled at middle of data output time

    //clock = Fosc/64
    SSPCON1bits.SSPM0 = 0;
    SSPCON1bits.SSPM1 = 1;
    SSPCON1bits.SSPM2 = 0;
    SSPCON1bits.SSPM3 = 0;


    SSPCON1bits.SSPEN = 1; //enable MSSP module
}

/**
 * @brief   Sends init settings to the SX1239
 *
 * @param   address SX1239 register address
 * @param   data Data to send
 * @return  received received data from SX1239
 */
uint8_t pictail_spi(uint8_t address, uint8_t data) {
    uint8_t received;

    PICtail_SS = 0;
    __delay_us(10);
    SSPBUF = address; //send address first
    while (!SSPSTATbits.BF); //wait until finished sending
    received = SSPBUF;
    SSPBUF = data;
    while (!SSPSTATbits.BF); //wait until finished sending
    received = SSPBUF;
    __delay_us(10);
    PICtail_SS = 1;

    return received;
}

uint8_t read_spi(uint8_t address)
{
    return pictail_spi(address & 0x7F, 0);
}