//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include "sx1239_config.h"
#include "bsp.h"

/**
 * @brief   Initilizies the SX1239 receiver in either packet or continous mode
 *      -Continous mode provides raw, demodulated data on the GPIO pins
 *      -Packet mode is for production where PAYLOADREADY is used as interrupt
 *       to determine when the transmission is ready to be emptied
 * @ref sx1239_config.h
 * @ref sx1239.h
 */
void sx1239_init(void) {
    SX1239_RESET = 1;
    SX1239_RESET_DIR = 0;
    __delay_ms(1);
    SX1239_RESET_DIR = 1;
    __delay_ms(10);
    
    // Set RegOpMode register
    pictail_spi(REGOPMODE_ADDRESS + WRITE, SX1239_OPMODE_ANS);

    // Set RegDataModul register
    pictail_spi(REGDATAMODUL_ADDRESS + WRITE, SX1239_DATAMODE_ANS);

    // Set RegFrfMsb, RegFrfMid, and RegFrfLsb registers
    pictail_spi(REGFRFMSB_ADDRESS + WRITE, (SX1239_FREQ_ANS & 0xFF0000) >> 16);
    pictail_spi(REGFRFMID_ADDRESS + WRITE, (SX1239_FREQ_ANS & 0x00FF00) >> 8);
    pictail_spi(REGFRFLSB_ADDRESS + WRITE, (SX1239_FREQ_ANS & 0x0000FF));

    // Set RegSyncConfig register
    pictail_spi(REGSYNCCONFIG_ADDRESS + WRITE, SX1239_SYNC_ANS);

    // Set the RegRxBw register
    pictail_spi(REGRXBW_ADDRESS + WRITE, SX1239_RXBW_ANS);

    // Set the RegRxThresh register
    pictail_spi(REGRSSITHRESH_ADDRESS + WRITE, SX1239_RSSI_THRESH_PEAK_ANS);

    // Set the RegOokPeak register
    pictail_spi(REGOOKPEAK_ADDRESS + WRITE, SX1239_OOK_PEAK_ANS);

    // Set the RegOokAvg register
    pictail_spi(REGOOKAVG_ADDRESS + WRITE, SX1239_THRESH_AVG_ANS);

    //GPIO_0 = SyncAddress, GPIO_1 = DCLK, GPIO_2 = DATA    --ONLY IN CONTINUOUS MODE!
    // Warning: must be defined after REGMODE/MODULATION
    pictail_spi(REGDIOMAPPING1_ADDRESS + WRITE, 0b00000000);

    while ( (read_spi( REGIRQFLAGS1_ADDRESS ) & 0x80 ) == 0x00 )
    {
      __delay_us(500);
    }
}

uint8_t sx1239_getVersion(void)
{
    return pictail_spi(REGVERSION_ADDRESS + READ, 0xAA); //get revision
}

/**
 * Optional CRC routine. Must pass in 1 byte at a time for a recurring
 * calculation. Uses CTCC-16 algorithm which the SX1239 uses as well
 */
int16_t crc_1021(uint8_t data, uint8_t crc)
{
    int16_t x;

    x = ((crc >> 8) ^ data) & 0xff;
    x ^= x >> 4;
    crc = (crc << 8) ^ (x << 12) ^ (x << 5) ^ x;
    crc &= 0xffff;
    return crc;
}
