//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include "Global.h"
#include "aes_keyGen.h"
#include "aes.h"
#include "keeloq.h"

uint8_t key1[16];
uint8_t block[16];

#define HopLo Buffer[15] //sync counter LSB
#define HopMi0 Buffer[14] //
#define HopMi1 Buffer[13] //sync counter
#define HopHi Buffer[12] // MSB

#define USER33 Buffer[8] //User bits
#define USER32 Buffer[9] //User bits
#define USER31 Buffer[10] //User bits
#define USER30 Buffer[11] //User bits
#define FUN1 Buffer[2] //Function

#define SN0 Buffer[7] //S/N LSB
#define SN1 Buffer[6] //S/N LSB
#define SN2 Buffer[5] //S/N
#define SN3 Buffer[4] //S/N MSB

//----------------------------------------------------------------------
//
// AES key Generation and decoding routine
//
// Normal Learn algorithm
//
// INPUT:  Serial Number (Buffer[16..19])
//             Manufacturer code 
// OUTPUT: key1[0..15] computed decryption key      
//
//----------------------------------------------------------------------

void AESKeyGen() {
    int8_t i;
    
    //Load the manufacturing key
    key1[0] = 0x11;
    key1[1] = 0x22;
    key1[2] = 0x33;
    key1[3] = 0x44;
    key1[4] = 0x55;
    key1[5] = 0x66;
    key1[6] = 0x77;
    key1[7] = 0x88;
    key1[8] = 0x99;
    key1[9] = 0x00;
    key1[10] = 0xAA;
    key1[11] = 0xBB;
    key1[12] = 0xCC;
    key1[13] = 0xDD;
    key1[14] = 0xEE;
    key1[15] = 0xFF;

    // Check if same Serial Number as last time while output active
    // it was stored in Seed
            // NOTE: THE FOLLOWING LINES (5 OFF ONLY) HAD SLASHES IN FRONT JASON JASON JASON CHANGED
 //    if (( SEED[0] != Buffer[19]) || // JASON JASON JASON
 //        ( SEED[1] != Buffer[18]) || // JASON JASON JASON
 //        ( SEED[2] != Buffer[17]) || // JASON JASON JASON
  //       ( SEED[3] != Buffer[16]) ) // JASON JASON JASON
  //    {   //If notthe same then generate key JASON JASON JASON

    // Compute LSB of Encryption key
    //  For this implementation the encryption key
    // was generated using 0xA5A5A5A5, 0x5A5A5A5A and 0x00000000
    //  as padding for the serial number.
    block[0] = 0xA5; // Padding for first part
    block[1] = 0xA5;
    block[2] = 0xA5;
    block[3] = 0xA5;
    block[4] = 0x5A; // Padding second part
    block[5] = 0x5A;
    block[6] = 0x5A;
    block[7] = 0x5A;
    block[8] = Buffer[19]; // Serial Number
    block[9] = Buffer[18];
    block[10] = Buffer[17];
    block[11] = Buffer[16];
    block[12] = 0x00; // Padding for last part
    block[13] = 0x00;
    block[14] = 0x00;
    block[15] = 0x00;

    AESCalcDecodeKey(&key1); // Perform key generation for Decode
    AESDecode(&block, &key1); // Decode

    for (i = 0; i < 16; i++) {
        key1[i] = block[i]; // Result is the encryption key to be used
    }

            // NOTE: THE FOLLOWING LINES (4 OFF ONLY) HAD SLASHES IN FRONT JASON JASON JASON CHANGED
 //   } // JASON JASON JASON
 //   else // same Serial Number as last time... JASON JASON JASON
   // {   // just keep on using same Decription Key JASON JASON JASON
   // } // JASON JASON JASON


    // Once having the KEY then we can go ahead and decrypt
    // The hopping coed is in Buffer 0-8 so we load this into hopcode[]

    // Load hopping code

    for (i = 0; i < 16; i++) {
        block[i] = Buffer[i]; // Load Received Hopping code into block
    }

    AESCalcDecodeKey(&key1); // Perform key generation for Decode
    AESDecode(&block, &key1); // Decode
    for (i = 0; i < 16; i++) {
        Buffer[i] = block[i]; // Decode result is now available
    }

}


/**
 *  Valid Decryption Check
 *
 * \return TRUE if decoded lsb serial number is equal to the plaintext lsb
 * Serial Number and decrypted function code is equal to the plain text
 * function code.
 *
 * \note User and discrimination checks can be added here.
 */
uint8_t AES_DecCHK(void) {

    if (SN0 != Buffer[16]) // compare LSB of Serial number
        return FALSE; //



    // verify function code
    if (((((Buffer[2] >> 4) & 0x0F)^ Buffer[20]) & 0x0F) != 0)
        return FALSE;

    return TRUE;
}

/**
 * Hopping Code Verification
 *
 * \return TRUE if hopping code is incrementing and inside a safe window
 */
uint8_t AES_ReqResync(void) {
    F2Chance = TRUE; // flag that a second (sequential) transmission
    NextHop = Hop + 1; // is needed to resynchronize receiver
    return FALSE; // cannot accept for now
}

/**
 *  Verify hopping code is within the acceptable window
 * \return TRUE if code is within acceptable window. Otherwise FALSE.
 */
uint8_t AES_HopCHK(void) {
    int16_t hopDiff;
    FHopOK = FALSE; // Hopping Code is not verified yet
    FSame = FALSE; // Hopping Code is not the same as previous

    // make it 2 16 bit signed integers 
    Hop1 = ((uint8_t) HopHi << 8) + HopMi1;
    Hop = ((uint8_t) HopMi0 << 8) + HopLo;

    if (F2Chance)
    {
        if (NextHop == Hop)
        {
            F2Chance = FALSE; // resync success
            FHopOK = TRUE;
            return TRUE;
        }
    }

    // verify EEPROM integrity
    if (ETemp != ETemp1)
        return AES_ReqResync(); // memory corrupted need a resync
    if (EHop != EHop1)
        return AES_ReqResync();

    // main comparison
    EHop = Hop1 - EHop1;

    if (EHop > 1)
        return FALSE; // This falls well within the locked region

    hopDiff = Hop - ETemp1; // subtract last value from new one

    if (hopDiff < 0)
    {   // locked region
        return FALSE; // fail
    }
    else if (hopDiff > 16)
    {   // resync region
        return AES_ReqResync();
    }
    else // 0>= ETemp >16 ; open window
    {
        if (hopDiff == 0) // same code (ETemp == 0)
            FSame = TRUE; // rise a flag

        FHopOK = TRUE;
        return TRUE;
    }
}

