//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include "includes.h"


//----------------------------------------------------------------------

void LoadManufCode() {
    DKEY[0] = 0xEF; // DKEY=0123456789ABCDEF
    DKEY[1] = 0xCD;
    DKEY[2] = 0xAB;
    DKEY[3] = 0x89;
    DKEY[4] = 0x67;
    DKEY[5] = 0x45;
    DKEY[6] = 0x23;
    DKEY[7] = 0x01;
}
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//
// Key Generation routine
//
// Normal Learn algorithm
//
// INPUT:  Serial Number (Buffer[4..7])
//         Manufacturer code 
// OUTPUT: DKEY[0..7] computed decryption key      
//
//----------------------------------------------------------------------
void NormalKeyGen()
{
	uint8_t HOPtemp[4];     // HOP temp buffer
	uint8_t SKEYtemp[4];    // Temp decryption key

        // Check if same Serial Number as last time while output active
        // it was stored in Seed 
        if (( SEED[0] != Buffer[4]) ||
            ( SEED[1] != Buffer[5]) ||
            ( SEED[2] != Buffer[6]) || 
            ( SEED[3] != (Buffer[7] & 0x0f)) ||
            (COut == 0))
        {   //If notthe same then generate key
            // NOTE: THE FOLLOWING LINE (1 OFF ONLY) HAD SLASHES IN FRONT JASON JASON JASON CHANGED
//            memcpy( (UINT16 *) HOPtemp, (UINT16 *) Buffer, 4);    // save hopping code to temp JASON JASON JASON
            memcpy( &HOPtemp, &Buffer, 4);

            memcpy( &SEED, &Buffer[4], 4);   	// make seed = Serial Number
            SEED[3] &= 0x0f;                		// mask out function codes       

            // compute LSb of decryption key first
            memcpy( &Buffer, &SEED, 4);       	// get SEED in
            Buffer[3] |= 0x20;            		// add constant 0x20, 
            LoadManufCode();           
            Decrypt();
            memcpy( &SKEYtemp, &Buffer, 4);   	// save result for later

            // compute MSb of decryption key 
            memcpy( &Buffer, &SEED, 4);       	// get SEED in
            Buffer[3] |= 0x60;            		// add constant 0x60, 
            LoadManufCode();
            Decrypt();
            memcpy( &DKEY[4], &Buffer, 4);   	// move it into DKEY MSb
            memcpy( &DKEY, &SKEYtemp, 4);     	// add LSb

            // ready for Decrypt
            memcpy( &Buffer, &HOPtemp, 4);    	// restore hopping code
        }
        else // same Serial Number as last time...
        {   // just keep on using same Decription Key
            NOP();
        } 
} 

//----------------------------------------------------------------------
//
// Valid Decryption Check
//
// INPUT:  Serial Number (Buffer[4..7])
//         Hopping Code  (Buffer[0..3])
// OUTPUT: TRUE if discrimination bits == lsb Serial Number
//              and decrypted function code == plain text function code 
//----------------------------------------------------------------------
uint8_t DecCHK(void)
{
             // NOTE: THE FOLLOWING LINES (4 OFF ONLY) HAD SLASHES IN FRONT JASON JASON JASON CHANGED
 // if ( DisLo != IDLo)    	 // compare low 8bit of Serial Number JASON JASON JASON
 //    return FALSE;		// verify discrimination bits JASON JASON JASON


 // if ( ( (Buffer[3] ^ IDMi) & 0x3)!= 0) // compare 9th and 10th bit of SN JASON JASON JASON
 //    return FALSE;// JASON JASON JASON

    // verify function code
    if ( ((Buffer[3] ^ Buffer[7]) & 0xf0)!= 0) // THESE ARE USUALLY ON!!!!!!JASON SWITCHED OFF
        return FALSE; //JASON SWITCHED OFF! THESE ARE USUALLY ON!!!!
    // JASON SWAP THE 2ND IF (WHICH IS OFF) FOR THE THIRD (WHIHC IS USUALLY ON)
    return TRUE;
} 

//----------------------------------------------------------------------
//
// Hopping Code Verification
//
// INPUT:  Hopping Code  (Buffer[0..3])
//         and previous value stored in EEPROM EHop
// OUTPUT: TRUE if hopping code is incrementing and inside a safe window (16)
//
//----------------------------------------------------------------------
uint8_t ReqResync(void)
{
    F2Chance= TRUE;         // flag that a second (sequential) transmission 
    NextHop = Hop+1;        // is needed to resynchronize receiver
    return FALSE;           // cannot accept for now
}

//----------------------------------------------------------------------
// HopCHK()
//  Verify hopping code is within the acceptable window
//
//----------------------------------------------------------------------
uint8_t HopCHK(void)
{
    int16_t hopDiff;
    FHopOK = FALSE;             // Hopping Code is not verified yet
    FSame = FALSE;              // Hopping Code is not the same as previous

    // make it a 16 bit signed integer 
    Hop = ((uint16_t)HopHi << 8) + HopLo;

    if ( F2Chance)
        if ( NextHop == Hop)
        {
            F2Chance = FALSE;       // resync success
            FHopOK = TRUE;       
            return TRUE;
        }
        
    // verify EEPROM integrity
    if ( EHop != ETemp)
        return ReqResync();         // memory corrupted need a resync
    
    // main comparison
    hopDiff = Hop - EHop;           // subtract last value from new one

    if ( hopDiff < 0)               // locked region
        return FALSE;               // fail

    else if ( hopDiff > 16)         // resync region
        return ReqResync();

    else                            // 0>= ETemp >16 ; open window
    {
        if ( hopDiff == 0)          // same code (ETemp == 0)
            FSame = TRUE;           // rise a flag

        FHopOK = TRUE;           
        return TRUE;
    }
} 

//----------------------------------------------------------------------
// End of File : KeyGen.C
//----------------------------------------------------------------------



